@@ -1,5 +1,5 @@
 /**
- *  Copyright 2002 Peter Seiderer <Peter.Seiderer@ciselant.de>
+ *  Copyright 2002-2005 Peter Seiderer <Peter.Seiderer@ciselant.de>
  *
  *  This file is part of SeBIE.
  *
@@ -225,15 +225,15 @@ load_pixbuf (struct sebie_s *sebie)
   scale1 = MIN(scale1, scale2);
   scale1 = MIN(scale1, scale3);
   scale1 = MIN(scale1, scale4);
-  printf("scale: %f \n", scale1);
+  sebie_debug("scale: %f \n", scale1);
   sebie->model.scaled_scale = scale1;
   sebie->model.scaled_width  = (int)((double)sebie->model.image_width * scale1);
   sebie->model.scaled_height = (int)((double)sebie->model.image_height * scale1);
   sebie->model.scaled_rotation = SEBIE_ROTATE_000;
 
-  printf("scaled: width: %d height: %d \n",
-	 sebie->model.scaled_width, 
-	 sebie->model.scaled_height);
+  sebie_debug("scaled: width: %d height: %d \n",
+	      sebie->model.scaled_width, 
+	      sebie->model.scaled_height);
   if (sebie->model.scaled_pixbuf_orig != NULL) {
     g_object_unref(sebie->model.scaled_pixbuf_orig);
   }
@@ -268,7 +268,7 @@ sebie_update_file_list(struct sebie_s *sebie)
 
   G_CONST_RETURN gchar *next;
   while((next = g_dir_read_name(gdir)) != NULL) {
-    printf("next %s\n", next);
+    sebie_debug("next %s\n", next);
 
     if (regexec(&sebie->model.regex, next, 0, NULL, 0) == 0) {
       sebie_file_list_add(sebie, next);
@@ -389,7 +389,7 @@ init_sebie_model(struct sebie_s *sebie)
   sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_FAC_H;
   sebie->dirty_tags |= SEBIE_UPDATE_IMAGE_GAMMA;
   
-  sebie->model.statusbar_text = strdup("   SeBIE  (C) 2003 by Peter.Seiderer@ciselant.de");
+  sebie->model.statusbar_text = strdup("   SeBIE  (C) 2002-2005 by Peter.Seiderer@ciselant.de");
   sebie->dirty_tags |= SEBIE_UPDATE_STATUS_BAR;
 }
 
