@@ -1,5 +1,5 @@
 /**
- *  Copyright 2002 Peter Seiderer <Peter.Seiderer@ciselant.de>
+ *  Copyright 2002-2005 Peter Seiderer <Peter.Seiderer@ciselant.de>
  *
  *  This file is part of SeBIE.
  *
@@ -64,6 +64,13 @@ typedef enum sebie_mouse_click_e {
   SEBIE_MOVE_SELECTION   = 2
 } sebie_mouse_click_type;
 
+typedef enum sebie_resize_corner_e {
+  SEBIE_UPPER_LEFT_CORNER,
+  SEBIE_UPPER_RIGHT_CORNER,
+  SEBIE_LOWER_RIGHT_CORNER,
+  SEBIE_LOWER_LEFT_CORNER
+} sebie_resize_corner_type;
+
 typedef struct sebie_view_s {
   GtkWidget *window1;
   GtkWidget *vbox1;
@@ -130,12 +137,9 @@ typedef struct sebie_model_s {
   gint image_grayscale;
   /* the selection */
   sebie_mouse_click_type mouse_click_type;
+  sebie_resize_corner_type resize_corner;
   gint mouse_pos_lx;
   gint mouse_pos_ly;
-  gint selection_lx;
-  gint selection_ly;
-  gint selection_lwidth;
-  gint selection_lheight;
   gint selection_x;
   gint selection_y;
   gint selection_width;
