@@ -1,5 +1,5 @@
 /**
- *  Copyright 2002 Peter Seiderer <Peter.Seiderer@ciselant.de>
+ *  Copyright 2002-2005 Peter Seiderer <Peter.Seiderer@ciselant.de>
  *
  *  This file is part of SeBIE.
  *
@@ -26,6 +26,8 @@
 #include <string.h>
 #include <stdlib.h>
 
+#define SEBIE_CATCH_PIXEL_COUNT  15
+
 
 void
 on_treeview_file_list_changed(GtkTreeSelection *selection,
@@ -47,7 +49,7 @@ on_treeview_file_list_changed(GtkTreeSelection *selection,
       for (i = 0; i < nmatch; i++) {
 	strncpy(buf_regex[i],sebie->model.input_file_name +  matchptr[i].rm_so, matchptr[i].rm_eo - matchptr[i].rm_so);
 	buf_regex[i][matchptr[i].rm_eo - matchptr[i].rm_so] = '\0';
-	printf("buf_regex[%d] = %s\n", i, buf_regex[i]);
+	sebie_debug("buf_regex[%d] = %s\n", i, buf_regex[i]);
       }
       char buf[256];
       snprintf(buf, 256, sebie->model.output_format, buf_regex[1]);
@@ -76,7 +78,7 @@ on_entry_input_regex_changed (GtkEditable *editable,
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
   G_CONST_RETURN gchar *input_regex = gtk_entry_get_text(GTK_ENTRY(editable));
-  printf("input_regex = %s\n", input_regex);
+  sebie_debug("input_regex = %s\n", input_regex);
   sebie_set_string(&sebie->model.input_regex, input_regex);
   sebie->model_tags |= SEBIE_MODEL_INPUT_REGEX;
   int err;
@@ -95,7 +97,7 @@ on_entry_output_format_changed (GtkEditable *editable,
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
   G_CONST_RETURN gchar *output_format = gtk_entry_get_text(GTK_ENTRY(editable));
-  printf("output_format = %s\n", output_format);
+  sebie_debug("output_format = %s\n", output_format);
   sebie_set_string(&sebie->model.output_format, output_format);
   sebie->model_tags |= SEBIE_MODEL_OUTPUT_FORMAT;
   save_sebie_config(sebie);
@@ -115,14 +117,14 @@ on_button_save_clicked (GtkButton *button,
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
 
-  printf("image_width:  %d\n", sebie->model.image_width);
-  printf("image_height: %d\n", sebie->model.image_height);
-  printf("image_gamma:  %lf\n", sebie->model.image_gamma);
-  printf("selection_x:  %d\n", sebie->model.selection_x);
-  printf("selection_y:  %d\n", sebie->model.selection_y);
-  printf("selection_width:  %d\n", sebie->model.selection_width);
-  printf("selection_height: %d\n", sebie->model.selection_height);
-  printf("scaled_scale: %lf\n", sebie->model.scaled_scale);
+  sebie_debug("image_width:  %d\n", sebie->model.image_width);
+  sebie_debug("image_height: %d\n", sebie->model.image_height);
+  sebie_debug("image_gamma:  %lf\n", sebie->model.image_gamma);
+  sebie_debug("selection_x:  %d\n", sebie->model.selection_x);
+  sebie_debug("selection_y:  %d\n", sebie->model.selection_y);
+  sebie_debug("selection_width:  %d\n", sebie->model.selection_width);
+  sebie_debug("selection_height: %d\n", sebie->model.selection_height);
+  sebie_debug("scaled_scale: %lf\n", sebie->model.scaled_scale);
   
   if (sebie->model.selection_width == 0 || 
       sebie->model.selection_height == 0) {
@@ -137,11 +139,11 @@ on_button_save_clicked (GtkButton *button,
   int fac_width = sebie->model.selection_fac_width;
   int fac_height = sebie->model.selection_fac_height;
 
-  printf("with rotation:\n");
-  printf("x:  %d\n", x);
-  printf("y:  %d\n", y);
-  printf("width:  %d\n", width);
-  printf("height: %d\n", height);
+  sebie_debug("with rotation:\n");
+  sebie_debug("x:  %d\n", x);
+  sebie_debug("y:  %d\n", y);
+  sebie_debug("width:  %d\n", width);
+  sebie_debug("height: %d\n", height);
 
   if (sebie->model.scaled_rotation == SEBIE_ROTATE_090) {
     gint tmp1 = x;
@@ -165,11 +167,11 @@ on_button_save_clicked (GtkButton *button,
     fac_width = sebie->model.selection_fac_height;
     fac_height = sebie->model.selection_fac_width;
   }
-  printf("without rotation:\n");
-  printf("x:  %d\n", x);
-  printf("y:  %d\n", y); 
-  printf("width:  %d\n", width);
-  printf("height: %d\n", height);
+  sebie_debug("without rotation:\n");
+  sebie_debug("x:  %d\n", x);
+  sebie_debug("y:  %d\n", y); 
+  sebie_debug("width:  %d\n", width);
+  sebie_debug("height: %d\n", height);
 
   if (x < 0 || y < 0 ||
       x + width > sebie->model.image_width ||
@@ -244,7 +246,7 @@ on_entry_selection_fac_width_changed (GtkEditable *editable,
   struct sebie_s *sebie = (struct sebie_s *)user_data;
   if (sebie->dirty_tags == 0) {
     G_CONST_RETURN gchar *fac_width = gtk_entry_get_text(GTK_ENTRY(editable));
-    printf("fac_width = %s %d\n", fac_width, atoi(fac_width));
+    sebie_debug("fac_width = %s %d\n", fac_width, atoi(fac_width));
     sebie->model.selection_fac_width = atoi(fac_width);
     sebie->model_tags |= SEBIE_MODEL_UPDATE_SELECTION_FAC;
     sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_FAC;
@@ -259,7 +261,7 @@ on_entry_selection_fac_height_changed (GtkEditable *editable,
   struct sebie_s *sebie = (struct sebie_s *)user_data;
   if (sebie->dirty_tags == 0) {
     G_CONST_RETURN gchar *fac_height = gtk_entry_get_text(GTK_ENTRY(editable));
-    printf("fac_height = %s %d\n", fac_height, atoi(fac_height));
+    sebie_debug("fac_height = %s %d\n", fac_height, atoi(fac_height));
     sebie->model.selection_fac_height = atoi(fac_height);
     sebie->model_tags |= SEBIE_MODEL_UPDATE_SELECTION_FAC;
     sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_FAC;
@@ -288,21 +290,21 @@ on_entry_selection_gamma_changed (GtkEditable *editable,
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
   G_CONST_RETURN gchar *gamma_str = gtk_entry_get_text(GTK_ENTRY(editable));
-  printf("image_gamma_str = '%s'\n", gamma_str);
+  sebie_debug("image_gamma_str = '%s'\n", gamma_str);
   double new_gamma = 0.0;
   if (sscanf(gamma_str, "%lf", &new_gamma) != 1) {
-    printf("no valid gamma input\n");
+    sebie_debug("no valid gamma input\n");
     return;
   }
   if (new_gamma < 0.1 || new_gamma > 3.0) {
-    printf("new_gamma %lf out of range\n", new_gamma);
+    sebie_debug("new_gamma %lf out of range\n", new_gamma);
     return;
   }
   if (new_gamma == sebie->model.image_gamma) {
-    printf("new_gamma %lf == old_gamma\n", new_gamma);
+    sebie_debug("new_gamma %lf == old_gamma\n", new_gamma);
     return;
   }
-  printf("set new image_gamma = %f\n", new_gamma);
+  sebie_debug("set new image_gamma = %f\n", new_gamma);
   sebie->model.image_gamma = new_gamma;
   sebie->model_tags |= SEBIE_MODEL_UPDATE_GAMMA;
   sebie->dirty_tags |= SEBIE_UPDATE_IMAGE_GAMMA;
@@ -315,7 +317,7 @@ on_button_grayscale_clicked(GtkButton *button,
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
   sebie->model.image_grayscale = !sebie->model.image_grayscale;
-  printf("set new image_grayscale = %d\n", sebie->model.image_grayscale);
+  sebie_debug("set new image_grayscale = %d\n", sebie->model.image_grayscale);
   sebie->model_tags |= SEBIE_MODEL_UPDATE_GRAYSCALE;
   apply_color_corrections(sebie);
 }
@@ -331,36 +333,110 @@ on_drawingarea1_expose_event (GtkWidget       *widget,
   return FALSE;
 }
 
+static gboolean
+in_selected_area(gint x, gint y, struct sebie_s *sebie)
+{
+  if (x >= sebie->model.selection_x &&
+      x <= sebie->model.selection_x + sebie->model.selection_width &&
+      y >= sebie->model.selection_y && 
+      y <= sebie->model.selection_y + sebie->model.selection_height) {
+    return TRUE;
+  } else {
+    return FALSE;
+  }
+}
+
+static gboolean
+in_upper_left_corner(gint x, gint y, struct sebie_s *sebie)
+{
+  if (x < sebie->model.selection_x &&
+      x > sebie->model.selection_x-SEBIE_CATCH_PIXEL_COUNT &&
+      y < sebie->model.selection_y &&
+      y > sebie->model.selection_y-SEBIE_CATCH_PIXEL_COUNT) {
+    return TRUE;
+  } else {
+    return FALSE;
+  }
+}
+
+static gboolean
+in_upper_right_corner(gint x, gint y, struct sebie_s *sebie)
+{
+  if (x > sebie->model.selection_x+sebie->model.selection_width &&
+      x < sebie->model.selection_x+sebie->model.selection_width+SEBIE_CATCH_PIXEL_COUNT &&
+      y < sebie->model.selection_y &&
+      y > sebie->model.selection_y-SEBIE_CATCH_PIXEL_COUNT) {
+    return TRUE;
+  } else {
+    return FALSE;
+  }
+}
+
+static gboolean
+in_lower_right_corner(gint x, gint y, struct sebie_s *sebie)
+{
+  if (x > sebie->model.selection_x+sebie->model.selection_width &&
+      x < sebie->model.selection_x+sebie->model.selection_width+SEBIE_CATCH_PIXEL_COUNT &&
+      y > sebie->model.selection_y+sebie->model.selection_height &&
+      y < sebie->model.selection_y+sebie->model.selection_height+SEBIE_CATCH_PIXEL_COUNT) {
+    return TRUE;
+  } else {
+    return FALSE;
+  }
+}
+
+static gboolean
+in_lower_left_corner(gint x, gint y, struct sebie_s *sebie)
+{
+  if (x < sebie->model.selection_x &&
+      x > sebie->model.selection_x-SEBIE_CATCH_PIXEL_COUNT &&
+      y > sebie->model.selection_y+sebie->model.selection_height &&
+      y < sebie->model.selection_y+sebie->model.selection_height+SEBIE_CATCH_PIXEL_COUNT) {
+    return TRUE;
+  } else {
+    return FALSE;
+  }
+}
+
 gboolean
-on_drawingarea1_button_press_event     (GtkWidget       *widget,
-                                        GdkEventButton  *event,
-                                        gpointer         user_data)
+on_drawingarea1_button_press_event (GtkWidget       *widget,
+				    GdkEventButton  *event,
+				    gpointer         user_data)
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
 
-  if (event->button == 1) {
-    // left button
-    //printf("buttion_press_event %f %f\n", event->x, event->y);
-    if (event->x > sebie->model.selection_x+sebie->model.selection_width-10 &&
-	event->x < sebie->model.selection_x+sebie->model.selection_width+10 &&
-	event->y > sebie->model.selection_y+sebie->model.selection_height-10 &&
-	event->y < sebie->model.selection_y+sebie->model.selection_height+10) {
+  if (event->button == 1) { /* left button */
+    sebie_debug("button_press_event %f %f\n", event->x, event->y);
+    if (in_upper_left_corner(event->x, event->y, sebie)) {
+      sebie_debug("UPPER_LEFT\n");
       sebie->model.mouse_click_type = SEBIE_RESIZE_SELECTION;
-    } else if (event->x > sebie->model.selection_x &&
-	event->x < sebie->model.selection_x + sebie->model.selection_width &&
-	event->y > sebie->model.selection_y &&
-	event->y < sebie->model.selection_y + sebie->model.selection_height)
-      {
-	sebie->model.mouse_click_type = SEBIE_MOVE_SELECTION;
-	sebie->model.mouse_pos_lx = event->x;
-	sebie->model.mouse_pos_ly = event->y;
-      } else {
-	sebie->model.mouse_click_type = SEBIE_RESIZE_SELECTION;
-	sebie->model.selection_x = event->x;
-	sebie->model.selection_y = event->y;
-	sebie->model.selection_width  = 0;
-	sebie->model.selection_height = 0;
-      }
+      sebie->model.resize_corner = SEBIE_UPPER_LEFT_CORNER;
+    } else if (in_upper_right_corner(event->x, event->y, sebie)) {
+      sebie_debug("UPPER_RIGHT\n");
+      sebie->model.mouse_click_type = SEBIE_RESIZE_SELECTION;
+      sebie->model.resize_corner = SEBIE_UPPER_RIGHT_CORNER;
+    } else if (in_lower_right_corner(event->x, event->y, sebie)) {
+      sebie_debug("LOWER_RIGHT\n");
+      sebie->model.mouse_click_type = SEBIE_RESIZE_SELECTION;
+      sebie->model.resize_corner = SEBIE_LOWER_RIGHT_CORNER;
+    } else if (in_lower_left_corner(event->x, event->y, sebie)) {
+      sebie_debug("LOWER_LEFT\n");
+      sebie->model.mouse_click_type = SEBIE_RESIZE_SELECTION;
+      sebie->model.resize_corner = SEBIE_LOWER_LEFT_CORNER;
+    } else if (in_selected_area(event->x, event->y, sebie)) {
+      sebie_debug("MOVE\n");
+      sebie->model.mouse_click_type = SEBIE_MOVE_SELECTION;
+      sebie->model.mouse_pos_lx = event->x;
+      sebie->model.mouse_pos_ly = event->y;
+    } else {
+      sebie_debug("LOWER_RIGHT NEW\n");
+      sebie->model.mouse_click_type = SEBIE_RESIZE_SELECTION;
+      sebie->model.resize_corner = SEBIE_LOWER_RIGHT_CORNER;
+      sebie->model.selection_x = event->x;
+      sebie->model.selection_y = event->y;
+      sebie->model.selection_width  = 0;
+      sebie->model.selection_height = 0;
+    }
     
     sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_RECT;
     sebie_update_view(sebie);
@@ -370,13 +446,14 @@ on_drawingarea1_button_press_event     (GtkWidget       *widget,
 }
 
 gboolean
-on_drawingarea1_motion_notify_event    (GtkWidget       *widget,
-                                        GdkEventMotion  *event,
-                                        gpointer         user_data)
+on_drawingarea1_motion_notify_event (GtkWidget       *widget,
+				     GdkEventMotion  *event,
+				     gpointer         user_data)
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
   
-  int x, y;
+  gint x;
+  gint y;
   GdkModifierType state;
   
   if (event->is_hint) {
@@ -386,22 +463,86 @@ on_drawingarea1_motion_notify_event    (GtkWidget       *widget,
     y = event->y;
     state = event->state;
   }
-  
-  if (state & GDK_BUTTON1_MASK) {
-    //printf("motion_notiry_event %d %d \n", x, y);
-    if (sebie->model.mouse_click_type == SEBIE_RESIZE_SELECTION) {
-      gint nwidth  = x - sebie->model.selection_x;
-      gint nheight = y - sebie->model.selection_y;
 
-      double nfrac = (double)nwidth / (double)nheight;
-      if (nfrac >= sebie->model.selection_fac) {
-	sebie->model.selection_width = (double)nheight * sebie->model.selection_fac;
-	sebie->model.selection_height = nheight;
-      } else {
-	sebie->model.selection_width = nwidth;
-	sebie->model.selection_height = (double)nwidth / sebie->model.selection_fac; 
+  if (state & GDK_BUTTON1_MASK) {
+    sebie_debug("motion_notify_event %d %d \n", x, y);
+    if (sebie->model.mouse_click_type == SEBIE_RESIZE_SELECTION) {
+      sebie_debug("resize selection\n");
+      gint x1 = sebie->model.selection_x;
+      gint y1 = sebie->model.selection_y;
+      gint x2 = sebie->model.selection_x + sebie->model.selection_width;
+      gint y2 = sebie->model.selection_y + sebie->model.selection_height;
+
+      if (sebie->model.resize_corner == SEBIE_UPPER_LEFT_CORNER) {
+	gint nwidth  = sebie->model.selection_x + sebie->model.selection_width - x;
+	gint nheight = sebie->model.selection_y + sebie->model.selection_height - y;
+	double nfrac = (double)nwidth / (double)nheight;
+	if (nfrac >= sebie->model.selection_fac) {
+	  sebie->model.selection_x = x2 - (double)nheight * sebie->model.selection_fac;
+	  sebie->model.selection_y = y;
+	  sebie->model.selection_width  = x2 - sebie->model.selection_x;
+	  sebie->model.selection_height = y2 - sebie->model.selection_y;
+	  
+	} else {
+	  sebie->model.selection_x = x;
+	  sebie->model.selection_y = y2 - (double)nwidth / sebie->model.selection_fac;
+	  sebie->model.selection_width = x2 - x;
+	  sebie->model.selection_height = y2 - sebie->model.selection_y;
+	}
+      } else if (sebie->model.resize_corner == SEBIE_UPPER_RIGHT_CORNER) {
+	gint nwidth  = x - sebie->model.selection_x;
+	gint nheight = sebie->model.selection_y + sebie->model.selection_height - y;
+	double nfrac = (double)nwidth / (double)nheight;
+	if (nfrac >= sebie->model.selection_fac) {
+	  sebie->model.selection_width = (double)nheight * sebie->model.selection_fac;
+	  sebie->model.selection_y = y;
+	  sebie->model.selection_height = y2 - sebie->model.selection_y;
+	} else {
+	  sebie->model.selection_width = nwidth;
+	  sebie->model.selection_height = (double)nwidth / sebie->model.selection_fac;
+	  sebie->model.selection_y = y2 - sebie->model.selection_height;
+	}
+      } else if (sebie->model.resize_corner == SEBIE_LOWER_RIGHT_CORNER) {
+	gint nwidth  = x - sebie->model.selection_x;
+	gint nheight = y - sebie->model.selection_y;
+	double nfrac = (double)nwidth / (double)nheight;
+	if (nfrac >= sebie->model.selection_fac) {
+	  sebie->model.selection_width = (double)nheight * sebie->model.selection_fac;
+	  sebie->model.selection_height = nheight;
+	} else {
+	  sebie->model.selection_width = nwidth;
+	  sebie->model.selection_height = (double)nwidth / sebie->model.selection_fac; 
+	}
+      } else { /* SEBIE_LOWER_LEFT_CORNER */
+	gint nwidth  = sebie->model.selection_x + sebie->model.selection_width - x;
+	gint nheight = y - sebie->model.selection_y;
+	double nfrac = (double)nwidth / (double)nheight;
+	if (nfrac >= sebie->model.selection_fac) {
+	  sebie->model.selection_width = (double)nheight * sebie->model.selection_fac;
+	  sebie->model.selection_height = nheight;
+	  sebie->model.selection_x = x2 - sebie->model.selection_width;
+	} else {
+	  sebie->model.selection_x = x;
+	  sebie->model.selection_height = (double)nwidth / sebie->model.selection_fac;
+	  sebie->model.selection_width = x2 - sebie->model.selection_x;
+	}
+      }
+      /* adjust if selection crossed the fix point */
+      if (sebie->model.selection_width < 0 &&
+	  sebie->model.selection_height < 0) {
+	sebie->model.selection_x += sebie->model.selection_width;
+	sebie->model.selection_width = sebie->model.selection_width * (-1);
+	sebie->model.selection_y += sebie->model.selection_height;
+	sebie->model.selection_height = sebie->model.selection_height * (-1);
+	switch(sebie->model.resize_corner) {
+	  case SEBIE_UPPER_LEFT_CORNER: sebie->model.resize_corner = SEBIE_LOWER_RIGHT_CORNER; break;
+	  case SEBIE_UPPER_RIGHT_CORNER: sebie->model.resize_corner = SEBIE_LOWER_LEFT_CORNER; break;
+	  case SEBIE_LOWER_RIGHT_CORNER: sebie->model.resize_corner = SEBIE_UPPER_LEFT_CORNER; break;
+	  case SEBIE_LOWER_LEFT_CORNER: sebie->model.resize_corner = SEBIE_UPPER_RIGHT_CORNER; break;
+	}
       }
     } else {
+      sebie_debug("move selection\n");
       gint diff_x = x - sebie->model.mouse_pos_lx;
       gint diff_y = y - sebie->model.mouse_pos_ly;
       sebie->model.selection_x += diff_x;
@@ -412,5 +553,33 @@ on_drawingarea1_motion_notify_event    (GtkWidget       *widget,
     sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_RECT;
     sebie_update_view(sebie);
   }
+
+  if (in_selected_area(x,y, sebie)) {
+    GdkCursor *cursor = gdk_cursor_new(GDK_FLEUR);
+    gdk_window_set_cursor(event->window, cursor);
+  } else if (in_upper_left_corner(x, y, sebie) ||
+	     ((state & GDK_BUTTON1_MASK) &&
+	      sebie->model.resize_corner == SEBIE_UPPER_LEFT_CORNER)) {
+    GdkCursor *cursor = gdk_cursor_new(GDK_TOP_LEFT_CORNER);
+    gdk_window_set_cursor(event->window, cursor);
+  } else if (in_upper_right_corner(x, y, sebie) ||
+	     ((state & GDK_BUTTON1_MASK) &&
+	      sebie->model.resize_corner == SEBIE_UPPER_RIGHT_CORNER)) {
+    GdkCursor *cursor = gdk_cursor_new(GDK_TOP_RIGHT_CORNER);
+    gdk_window_set_cursor(event->window, cursor);
+  } else if (in_lower_right_corner(x, y, sebie) ||
+	     ((state & GDK_BUTTON1_MASK) &&
+	      sebie->model.resize_corner == SEBIE_LOWER_RIGHT_CORNER)) {
+    GdkCursor *cursor = gdk_cursor_new(GDK_BOTTOM_RIGHT_CORNER);
+    gdk_window_set_cursor(event->window, cursor);
+  } else if (in_lower_left_corner(x, y, sebie) ||
+	     ((state & GDK_BUTTON1_MASK) &&
+	      sebie->model.resize_corner == SEBIE_LOWER_LEFT_CORNER)) {
+    GdkCursor *cursor = gdk_cursor_new(GDK_BOTTOM_LEFT_CORNER);
+    gdk_window_set_cursor(event->window, cursor);
+  } else {
+    gdk_window_set_cursor (event->window, NULL);
+  }
+
   return TRUE;
 }
