@@ -83,9 +83,9 @@ rotate_pixbuf(GdkPixbuf *pixbuf, int rotation)
 }
 
 GdkPixbuf *
-pixbuf_color_corrections(GdkPixbuf *pixbuf, double gamma)
+pixbuf_color_corrections(GdkPixbuf *pixbuf, double gamma, int grayscale)
 {
-  if (gamma == 1.0) {
+  if (gamma == 1.0 && grayscale == 0) {
     return gdk_pixbuf_copy(pixbuf);
   } else {
     GdkPixbuf *corrected = gdk_pixbuf_new(gdk_pixbuf_get_colorspace(pixbuf),
@@ -94,7 +94,7 @@ pixbuf_color_corrections(GdkPixbuf *pixbuf, double gamma)
 					  gdk_pixbuf_get_width(pixbuf),
 					  gdk_pixbuf_get_height(pixbuf));
     
-    int bits_per_sample = gdk_pixbuf_get_bits_per_sample(pixbuf);
+    //int bits_per_sample = gdk_pixbuf_get_bits_per_sample(pixbuf);
     int width  = gdk_pixbuf_get_width(pixbuf);;
     int height = gdk_pixbuf_get_height(pixbuf);
     int rowstride_o = gdk_pixbuf_get_rowstride(pixbuf);
@@ -126,6 +126,13 @@ pixbuf_color_corrections(GdkPixbuf *pixbuf, double gamma)
 	corr[i * channels + j * rowstride_c + 0] = gamma_lut[orig[i * channels + j * rowstride_o + 0]];
 	corr[i * channels + j * rowstride_c + 1] = gamma_lut[orig[i * channels + j * rowstride_o + 1]];
 	corr[i * channels + j * rowstride_c + 2] = gamma_lut[orig[i * channels + j * rowstride_o + 2]];
+	if (grayscale) {
+	  guint grayvalue = (corr[i * channels + j * rowstride_c + 0] + corr[i * channels + j * rowstride_c + 1] + corr[i * channels + j * rowstride_c + 2]) / 3;
+	  corr[i * channels + j * rowstride_c + 0] = grayvalue;
+	  corr[i * channels + j * rowstride_c + 1] = grayvalue;
+	  corr[i * channels + j * rowstride_c + 2] = grayvalue;
+	  
+	}
       }
     }
     g_free(gamma_lut);
