@@ -27,6 +27,6 @@ GdkPixbuf *
 rotate_pixbuf(GdkPixbuf *pixbuf, int rotation);
 
 GdkPixbuf *
-pixbuf_color_corrections(GdkPixbuf *pixbuf, double gamma);
+pixbuf_color_corrections(GdkPixbuf *pixbuf, double gamma, int grayscale);
 
 #endif /* _SEBIE_VIEW_H_ */
