@@ -1,5 +1,5 @@
 /**
- *  Copyright 2002 Peter Seiderer <Peter.Seiderer@ciselant.de>
+ *  Copyright 2002-2005 Peter Seiderer <Peter.Seiderer@ciselant.de>
  *
  *  This file is part of SeBIE.
  *
@@ -26,4 +26,10 @@
 void
 sebie_set_string(gchar **old_text, const gchar *new_text);
 
+void
+sebie_error(const char *format, ...);
+
+void
+sebie_debug(const char *format, ...);
+
 #endif /* _SEBIE_HELPERS_H_ */
